;################################################################################
;#										#
;# chipbasic - single chip basic computer with ATMega 32			#
;# monitor module								#
;# copyright (c) 2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-----------------------------------------------------------------------
; backup and restore screen
;-----------------------------------------------------------------------
mon_backup:	lds	XL,libmio_color
		sts	libmio_ram+52,XL
		lds	XL,libmio_channel
		sts	libmio_ram+53,XL
		sts	libmio_channel,byte0	;screen
	    	ldi	ZL,LOW(libmio_vram)	;source address
		ldi	ZH,HIGH(libmio_vram)
		ldi	YL,LOW(mon_bpage)	;dest address
		ldi	YH,HIGH(mon_bpage)
mon_backup0:	ldi	XH,libmio_rows
mon_backup1:	ldi	XL,libmio_cols
mon_backup2:	ld	tempreg1,Z+		;get byte
		st	Y+,tempreg1
		dec	XL
		brne	mon_backup2
		dec	XH
		brne	mon_backup1
		ret
		
mon_restore:	lds	XL,libmio_ram+52
		sts	libmio_color,XL
		lds	XL,libmio_ram+53
		sts	libmio_channel,XL
		ldi	ZL,LOW(mon_bpage)	;source address
		ldi	ZH,HIGH(mon_bpage)
		ldi	YL,LOW(libmio_vram)	;dest address
		ldi	YH,HIGH(libmio_vram)
		rjmp	mon_backup0		

;-----------------------------------------------------------------------
; show io
;-----------------------------------------------------------------------
mon_sio:	ldi	YL,LOW(libmio_vram+19*libmio_cols+4)
		ldi	YH,HIGH(libmio_vram+19*libmio_cols+4)
		ldi	XL,0x09			;in
		ldi	XH,0x08			;out
		in	tempreg1,DDRA
		ldi	tempreg2,8
mon_sio1:	sbrc	tempreg1,0
		st	Y+,XL
		sbrs	tempreg1,0
		st	Y+,XH
		lsr	tempreg1
		dec	tempreg2
		brne	mon_sio1
		ldi	YL,LOW(libmio_vram+20*libmio_cols+4)
		ldi	YH,HIGH(libmio_vram+20*libmio_cols+4)
		ldi	XL,0x0b			;0
		ldi	XH,0x0a			;1
		in	tempreg1,PINA
		ldi	tempreg2,8
mon_sio2:	sbrc	tempreg1,0
		st	Y+,XL
		sbrs	tempreg1,0
		st	Y+,XH
		lsr	tempreg1
		dec	tempreg2
		brne	mon_sio2
		ret

;-----------------------------------------------------------------------
; show variables
;-----------------------------------------------------------------------
mon_svar:	mov	tempreg1,tempreg2
		subi	tempreg1,0xbf		;+65
		libmio_outchar			;write var name
		ldi	tempreg1,'='
		libmio_outchar
		ldi	YL,LOW(varspace)
		ldi	YH,HIGH(varspace)
		add	YL,tempreg2
		adc	YH,byte0
		add	YL,tempreg2
		adc	YH,byte0
		ld	XL,Y+
		ld	XH,Y+
		libmio_outdez
		ret

;-----------------------------------------------------------------------
; show stack
;-----------------------------------------------------------------------
mon_viewstack:	ldi	XL,19
		libmio_gotoxy
		ldi	XL,5
		mov	YL,tempreg3
		dec	YL
		mul	XL,YL
		ldi	YL,LOW(bas_stacks)
		ldi	YH,HIGH(bas_stacks)
		add	YL,r0
		adc	YH,r1
		ld	XL,Y			;line number
		sbrs	XL,7			;check call/for flag
		rjmp	mon_vst_10		;for
		libmio_thistext
		.db	255,"CALL  ",0
		ldd	XL,Y+1			;statement number
		andi	XL,0x7f
		rjmp	mon_vst_11

		ret
mon_vst_10:	libmio_thistext
		.db	255,"FOR ",0
		ldd	tempreg1,Y+2		;variable offset
		lsr	tempreg1
		subi	tempreg1,0xbf
		libmio_outchar
		libmio_outspace
		ldd	XL,Y+1			;line number
mon_vst_11:	clr	XH
		inc	XL
    		ldi	ctrl,2			;min with zeroes
		libmio_outdez			;write line number
		ldi	tempreg1,':'
		libmio_outchar
		clr	XH
		ld	XL,Y
		libmio_outdez			;write statement number
		ret

;-----------------------------------------------------------------------
; show monitor screen
;-----------------------------------------------------------------------
mon_show:	libmio_clrscr			;clear screen
		libmio_thistext
		.db 18,0,"PIN 12345678",0	;io view
		libmio_thistext
		.db 19,0,"DIR",0		;io view
		libmio_thistext
		.db 20,0,"LEV",0		;io view
		libmio_thistext
		.db 21,0,0			;set cursor
		ldi	ZL,libmio_cols
		ldi	tempreg1,0x0c
		libmio_mtext
		libmio_thistext
		.db 22,0,0x03,0x04,"o ",0x03,0x05,"tep ",0x03,0x06,"iew  e",0x07,0x02,"t",0	;bottom message
		libmio_thistext
		.db 0,0,"CHIPBASIC MONITOR     P",0	;head message
		lds	tempreg1,libmio_ram+51
		subi	tempreg1,0xcf
		libmio_outchar
		libmio_thistext
		.db 255,":",0		
		ldi 	ctrl,2			;format 2 digits
		clr	XH
		mov	XL,r12			;line number
		inc	XL
		libmio_outdez
		libmio_thistext
		.db 255,":",0		
		clr	XH
		mov	XL,r13			;statement number
		inc	XL
		libmio_outdez
		ldi	ZL,LOW(bas_inbuf+3)	;copy line
		ldi	ZH,HIGH(bas_inbuf+3)
		ldi	YL,LOW(libmio_vram+1*libmio_cols)
		ldi	YH,HIGH(libmio_vram+1*libmio_cols)
		ldi	XL,libmio_cols
		ldi	tempreg1,0x0c
mon_show0:	ld	XH,Z+
		cpi	XH,0x00			;end of line
		breq	mon_show00
		std	Y+libmio_cols,XH
		std	Y+(2*libmio_cols),tempreg1
		st	Y+,tempreg1
		dec	XL
		brne	mon_show0		
		rjmp	mon_show04
mon_show00:	ldi	XH,0x20
		std	Y+libmio_cols,XH
		std	Y+(2*libmio_cols),tempreg1
		st	Y+,tempreg1
		dec	XL
		brne	mon_show00		
mon_show04:	
	
		ldi 	ctrl,0x0d		;format 5 digits/leading spaces 
		ldi	tempreg2,0x00		;start with var A
mon_show1:	mov	tempreg1,tempreg2	;copy number
		subi	tempreg1,252		;+4
		mov	XH,tempreg1
		clr	XL
		libmio_gotoxy
		rcall	mon_svar
		inc	tempreg2
		cpi	tempreg2,13
		brne	mon_show1		
		
mon_show2:	mov	tempreg1,tempreg2	;copy number
		subi	tempreg1,9		;-13+4
		mov	XH,tempreg1
		ldi	XL,9
		libmio_gotoxy
		rcall	mon_svar
		inc	tempreg2
		cpi	tempreg2,26
		brne	mon_show2		

		libmio_thistext
		.db	4,19,"STACK:",0
		clr ctrl			;format 2 digits compact
		clr	XH
		lds	XL,bas_ram+1		;stackpointer
		libmio_outdez

		ldi	tempreg3,12

mon_show4:	lds	YL,bas_ram+1			;stackpointer
		cp	YL,tempreg3
		brcs	mon_show5
		sub	YL,tempreg3
		ldi	XH,4
		add	XH,tempreg3
		rcall	mon_viewstack
mon_show5:	dec	tempreg3
		brne	mon_show4


mon_show8:	ret



;-----------------------------------------------------------------------
; monitor main loop
;-----------------------------------------------------------------------
mon_main:	push	tempreg1
	    	push	tempreg2
		push	tempreg3
	    	push	tempreg4
		push	ZL
		push	ZH
		push	YL
		push	YH
		push	XL
		push	XH
		push	libmio_cur_x
		push	libmio_cur_y
		push	ereg
		push	ctrl

		rcall	mon_backup		;save screen
		rcall	lce_xpand		;expand line
mon_main_00:	rcall	mon_show		;show screen
mon_main_0:	rcall	mon_waitkey		;wait for key
		clr	ereg
		lds	XL,libmio_kflags
		cpi	tempreg1,'x'		;exit
		brne	mon_main_1
		ori	XL,0x01			;set break flag
		rjmp	mon_main_end

mon_main_1:	cpi	tempreg1,'g'		;go
		brne	mon_main_2
		clr	XL			;continue
		rjmp	mon_main_end

mon_main_2:	cpi	tempreg1,'s'		;step
		brne	mon_main_3 
		ori	XL,0x04			;set monitor flag again
		rjmp	mon_main_end1

mon_main_3:	cpi	tempreg1,'v'		;view
		brne	mon_main_0
		rcall	mon_restore		;restore screen
		libmio_waitkey			;wait for keypress
		rjmp	mon_main_00		;monitor loop	
		

mon_main_end:	sts	libmio_kflags,XL	;store flags	
mon_main_end1:	lds	tempreg1,libmio_keycode	;wait until key is released
		cpi	tempreg1,0
		brne	mon_main_end1
		rcall	mon_restore
mme1:		pop	ctrl
		pop	ereg
		pop	libmio_cur_y
		pop	libmio_cur_x
		pop	XH
		pop	XL
		pop	YH
		pop	YL
		pop	ZH
		pop	ZL
		pop	tempreg4
		pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		ret	
							
mon_waitkey:	sts	libmio_lastkey,byte0	;delete last pressed key
mon_waitkey1:	rcall	mon_sio			;show IO
		lds	tempreg1,libmio_lastkey
		cpi	tempreg1,0
		breq	mon_waitkey1
		jmp	libmio_scrs		;screenshot
		